@@ -28,7 +28,8 @@ from .errorhandler import ErrorHandler
 from .switch_to import SwitchTo
 from .mobile import Mobile
 from .file_detector import FileDetector, LocalFileDetector
-from selenium.common.exceptions import WebDriverException
+from selenium.common.exceptions import (InvalidArgumentException,
+                                        WebDriverException)
 from selenium.webdriver.common.by import By
 from selenium.webdriver.common.html5.application_cache import ApplicationCache
 
@@ -80,6 +81,8 @@ class WebDriver(object):
         if not isinstance(desired_capabilities, dict):
             raise WebDriverException("Desired Capabilities must be a dictionary")
         if proxy is not None:
+            warnings.warn("Please use FirefoxOptions to set proxy",
+                          DeprecationWarning)
             proxy.add_to_capabilities(desired_capabilities)
         self.command_executor = command_executor
         if type(self.command_executor) is bytes or isinstance(self.command_executor, str):
@@ -89,6 +92,9 @@ class WebDriver(object):
         self.capabilities = {}
         self.error_handler = ErrorHandler()
         self.start_client()
+        if browser_profile is not None:
+            warnings.warn("Please use FirefoxOptions to set browser profile",
+                          DeprecationWarning)
         self.start_session(desired_capabilities, browser_profile)
         self._switch_to = SwitchTo(self)
         self._mobile = Mobile(self)
@@ -157,7 +163,7 @@ class WebDriver(object):
         """
         pass
 
-    def start_session(self, desired_capabilities, browser_profile=None):
+    def start_session(self, capabilities, browser_profile=None):
         """
         Creates a new session with the desired capabilities.
 
@@ -168,21 +174,26 @@ class WebDriver(object):
          - javascript_enabled - Whether the new session should support JavaScript.
          - browser_profile - A selenium.webdriver.firefox.firefox_profile.FirefoxProfile object. Only used if Firefox is requested.
         """
-        capabilities = {'desiredCapabilities': {}, 'requiredCapabilities': {}}
-        for k, v in desired_capabilities.items():
-            if k not in ('desiredCapabilities', 'requiredCapabilities'):
-                capabilities['desiredCapabilities'][k] = v
-            else:
-                capabilities[k].update(v)
+        if not isinstance(capabilities, dict):
+            raise InvalidArgumentException("Capabilities must be a dictionary")
+        w3c_caps = {"firstMatch": [], "alwaysMatch": {}}
+        w3c_caps.update(capabilities)
         if browser_profile:
-            capabilities['desiredCapabilities']['firefox_profile'] = browser_profile.encoded
-        response = self.execute(Command.NEW_SESSION, capabilities)
+            w3c_caps["firstMatch"].append({"firefox_profile": browser_profile.encoded})
+        parameters = {"capabilities": w3c_caps,
+                      "desiredCapabilities": capabilities}
+        response = self.execute(Command.NEW_SESSION, parameters)
         if 'sessionId' not in response:
             response = response['value']
         self.session_id = response['sessionId']
-        self.capabilities = response['value']
+        self.capabilities = response.get('value')
 
-        # Quick check to see if we have a W3C Compliant browser
+        # if capabilities is none we are probably speaking to
+        # a W3C endpoint
+        if self.capabilities is None:
+            self.capabilities = response.get('capabilities')
+
+        # Double check to see if we have a W3C Compliant browser
         self.w3c = response.get('status') is None
 
     def _wrap_value(self, value):
@@ -462,7 +473,13 @@ class WebDriver(object):
             driver.execute_script('document.title')
         """
         converted_args = list(args)
-        return self.execute(Command.EXECUTE_SCRIPT, {
+        command = None
+        if self.w3c:
+            command = Command.W3C_EXECUTE_SCRIPT
+        else:
+            command = Command.EXECUTE_SCRIPT
+
+        return self.execute(command, {
             'script': script,
             'args': converted_args})['value']
 
@@ -478,7 +495,12 @@ class WebDriver(object):
             driver.execute_async_script('document.title')
         """
         converted_args = list(args)
-        return self.execute(Command.EXECUTE_ASYNC_SCRIPT, {
+        if self.w3c:
+            command = Command.W3C_EXECUTE_SCRIPT_ASYNC
+        else:
+            command = Command.EXECUTE_ASYNC_SCRIPT
+
+        return self.execute(command, {
             'script': script,
             'args': converted_args})['value']
 
@@ -531,7 +553,10 @@ class WebDriver(object):
         :Usage:
             driver.current_window_handle
         """
-        return self.execute(Command.GET_CURRENT_WINDOW_HANDLE)['value']
+        if self.w3c:
+            return self.execute(Command.W3C_GET_CURRENT_WINDOW_HANDLE)['value']
+        else:
+            return self.execute(Command.GET_CURRENT_WINDOW_HANDLE)['value']
 
     @property
     def window_handles(self):
@@ -541,7 +566,10 @@ class WebDriver(object):
         :Usage:
             driver.window_handles
         """
-        return self.execute(Command.GET_WINDOW_HANDLES)['value']
+        if self.w3c:
+            return self.execute(Command.W3C_GET_WINDOW_HANDLES)['value']
+        else:
+            return self.execute(Command.GET_WINDOW_HANDLES)['value']
 
     def maximize_window(self):
         """
@@ -722,13 +750,13 @@ class WebDriver(object):
         :Usage:
             driver.set_page_load_timeout(30)
         """
-        if self.w3c:
-          self.execute(Command.SET_TIMEOUTS, {
-              'pageLoad': int(float(time_to_wait) * 1000)})
-        else:
-          self.execute(Command.SET_TIMEOUTS, {
-              'ms': float(time_to_wait) * 1000,
-              'type': 'page load'})
+        try:
+            self.execute(Command.SET_TIMEOUTS, {
+                'pageLoad': int(float(time_to_wait) * 1000)})
+        except WebDriverException:
+            self.execute(Command.SET_TIMEOUTS, {
+                'ms': float(time_to_wait) * 1000,
+                'type': 'page load'})
 
     def find_element(self, by=By.ID, value=None):
         """
@@ -903,6 +931,32 @@ class WebDriver(object):
             return self.execute(Command.GET_WINDOW_POSITION, {
                 'windowHandle': windowHandle})['value']
 
+    def get_window_rect(self):
+        """
+        Gets the x, y coordinates of the window as well as height and width of
+        the current window.
+
+        :Usage:
+            driver.get_window_rect()
+        """
+        return self.execute(Command.GET_WINDOW_RECT)['value']
+
+    def set_window_rect(self, x=None, y=None, width=None, height=None):
+        """
+        Sets the x, y coordinates of the window as well as height and width of
+        the current window.
+
+        :Usage:
+            driver.set_window_rect(x=10, y=10)
+            driver.set_window_rect(width=100, height=200)
+            driver.set_window_rect(x=10, y=10, width=100, height=200)
+        """
+        if (x is None and y is None) and (height is None and width is None):
+            raise InvalidArgumentException("x and y or height and width need values")
+
+        return self.execute(Command.SET_WINDOW_RECT, {"x": x, "y": y,
+                                                      "width": width,
+                                                      "height": height})['value']
     @property
     def file_detector(self):
         return self._file_detector
